@@ -14,7 +14,6 @@
 
 """The Coordinator is used to control the data flow between `Worker` and `Server`."""
 
-import logging
 import os
 import signal
 import socket
@@ -24,12 +23,13 @@ import traceback
 from multiprocessing.synchronize import Event
 from typing import Any, Callable, Optional, Sequence, Tuple, Type
 
-from .errors import DecodingError, ValidationError
+from .errors import MosecError
 from .ipc import IPCWrapper
-from .protocol import Protocol
+from .log import get_logger
+from .protocol import HTTPStautsCode, Protocol
 from .worker import Worker
 
-logger = logging.getLogger(__name__)
+logger = get_logger()
 
 
 CONN_MAX_RETRY = 10
@@ -118,7 +118,7 @@ class Coordinator:
                 return
             retry_count += 1
             logger.debug(
-                "%s trying to find the socket file: %s (%d/%d)",
+                "%s is trying to find the socket file: %s (%d/%d)",
                 self.name,
                 self.protocol.addr,
                 retry_count,
@@ -128,9 +128,10 @@ class Coordinator:
             continue
 
         logger.error(
-            "%s cannot find the socket file: %s", self.name, self.protocol.addr
+            "%s cannot find the socket file: %s, will shutdown",
+            self.name,
+            self.protocol.addr,
         )
-        logger.info("%s exiting...", self.name)
 
     def warmup(self):
         """Warmup to allocate resources (useful for GPU workload)[Optional]."""
@@ -144,7 +145,7 @@ class Coordinator:
                 for i, example in enumerate(self.worker.multi_examples):
                     self.worker.forward(example)
                     logger.debug(
-                        "Warming up... (%d / %d)",
+                        "warming up... (%d/%d)",
                         i + 1,
                         num_eg,
                     )
@@ -154,7 +155,7 @@ class Coordinator:
         # pylint: disable=broad-except
         except Exception:
             logger.error(
-                "%s warmup failed: %s\nplease ensure"
+                "%s warmup failed: %s, please ensure"
                 " worker's example meets its forward input format",
                 self.name,
                 traceback.format_exc().replace("\n", " "),
@@ -223,7 +224,7 @@ class Coordinator:
 
         # TODO(kemingy) find a better way
         def wrapped_send(flag: int, ids: Sequence[bytes], payloads: Sequence[bytes]):
-            if flag == Protocol.FLAG_OK:
+            if flag == HTTPStautsCode.OK:
                 payloads = self.ipc_wrapper.put(payloads)  # type: ignore
             return self.protocol.send(flag, ids, payloads)
 
@@ -242,10 +243,11 @@ class Coordinator:
                 continue
             except (struct.error, OSError) as err:
                 if not self.shutdown_notify.is_set():
-                    logger.error("%s socket receive error: %s", self.name, err)
+                    logger.error("%s failed to read from socket: %s", self.name, err)
                 break
 
             # pylint: disable=broad-except
+            length = len(payloads)
             try:
                 data = [decoder(item) for item in payloads]
                 data = (
@@ -253,34 +255,28 @@ class Coordinator:
                     if self.worker.max_batch_size > 1
                     else (self.worker.forward(data[0]),)
                 )
-                if len(data) != len(payloads):
+                if len(data) != length:
                     raise ValueError(
                         "returned data size doesn't match the input data size:"
-                        f"input({len(data)})!=output({len(payloads)})"
+                        f"input({length})!=output({len(data)})"
                     )
-                status = self.protocol.FLAG_OK
+                status = HTTPStautsCode.OK
                 payloads = [encoder(item) for item in data]
-            except DecodingError as err:
+            except MosecError as err:
                 err_msg = str(err).replace("\n", " - ")
-                err_msg = err_msg if err_msg else "cannot deserialize request bytes"
-                logger.info("%s decoding error: %s", self.name, err_msg)
-                status = self.protocol.FLAG_BAD_REQUEST
-                payloads = (f"decoding error: {err_msg}".encode(),)
-            except ValidationError as err:
-                err_msg = str(err)
-                err_msg = err_msg if err_msg else "invalid data format"
-                logger.info("%s validation error: %s", self.name, err_msg)
-                status = self.protocol.FLAG_VALIDATION_ERROR
-                payloads = (f"validation error: {err_msg}".encode(),)
+                err_msg = err_msg if err_msg else err.msg
+                logger.info("%s %s: %s", self.name, err.msg, err_msg)
+                status = err.code
+                payloads = [f"{err.msg}: {err_msg}".encode()] * length
             except Exception:
                 logger.warning(traceback.format_exc().replace("\n", " "))
-                status = self.protocol.FLAG_INTERNAL_ERROR
-                payloads = ("inference internal error".encode(),)
+                status = HTTPStautsCode.INTERNAL_ERROR
+                payloads = ["inference internal error".encode()] * length
 
             try:
                 protocol_send(status, ids, payloads)
             except OSError as err:
-                logger.error("%s socket send error: %s", self.name, err)
+                logger.error("%s failed to send to socket: %s", self.name, err)
                 break
 
         self.protocol.close()
