@@ -16,12 +16,30 @@
 
 This module provides a way to define the service components for machine learning
 model serving.
+
+Dynamic Batching
+================
+
+    The user may enable the dynamic batching feature for any stage when the
+    corresponding worker is appended, by setting the
+    :py:meth:`append_worker(max_batch_size) <Server.append_worker>`.
+
+Multiprocess
+============
+
+    The user may spawn multiple processes for any stage when the
+    corresponding worker is appended, by setting the
+    :py:meth:`append_worker(num) <Server.append_worker>`.
+
+IPC Wrapper
+===========
+
+    The user may wrap the inter-process communication to use shared memory,
+    e.g. pyarrow plasma, by providing the :py:mod:`IPC Wrapper <mosec.ipc.IPCWrapper>`
+    for the server.
 """
 
-import contextlib
-import logging
 import multiprocessing as mp
-import os
 import shutil
 import signal
 import subprocess
@@ -34,12 +52,15 @@ from typing import Dict, List, Optional, Type, Union
 
 import pkg_resources
 
-from .args import parse_arguments
+from .args import mosec_args
 from .coordinator import STAGE_EGRESS, STAGE_INGRESS, Coordinator
+from .dry_run import DryRunner
+from .env import env_var_context
 from .ipc import IPCWrapper
+from .log import get_logger
 from .worker import Worker
 
-logger = logging.getLogger(__name__)
+logger = get_logger()
 
 
 GUARD_CHECK_INTERVAL = 1
@@ -51,18 +72,6 @@ class Server:
 
     It allows users to sequentially append workers they implemented, builds
     the workflow pipeline automatically and starts up the server.
-
-    ###### Batching
-    > The user may enable the batching feature for any stage when the
-    corresponding worker is appended, by setting the `max_batch_size`.
-
-    ###### Multiprocess
-    > The user may spawn multiple processes for any stage when the
-    corresponding worker is appended, by setting the `num`.
-
-    ###### IPC Wrapper
-    > The user may wrap the inter-process communication to use shared memory,
-    e.g. pyarrow plasma, by providing the IPC wrapper for the server.
     """
 
     # pylint: disable=too-many-instance-attributes
@@ -73,14 +82,14 @@ class Server:
         """Initialize a MOSEC Server.
 
         Args:
-            ipc_wrapper (Optional[Union[IPCWrapper, partial]], optional):
-                IPCWrapper class. Defaults to None.
+            ipc_wrapper: wrapper function (before and after) IPC
         """
         self.ipc_wrapper = ipc_wrapper
 
         self._worker_cls: List[Type[Worker]] = []
         self._worker_num: List[int] = []
         self._worker_mbs: List[int] = []
+        self._worker_wait: List[int] = []
 
         self._coordinator_env: List[Union[None, List[Dict[str, str]]]] = []
         self._coordinator_ctx: List[str] = []
@@ -92,9 +101,11 @@ class Server:
 
         self._daemon: Dict[str, Union[subprocess.Popen, mp.Process]] = {}
 
-        self._configs: dict = {}
+        self._configs: dict = vars(mosec_args)
 
         self._server_shutdown: bool = False
+
+    def _handle_signal(self):
         signal.signal(signal.SIGTERM, self._terminate)
         signal.signal(signal.SIGINT, self._terminate)
 
@@ -109,14 +120,15 @@ class Server:
         worker,
         num,
         max_batch_size,
+        max_wait_time,
         start_method,
         env,
     ):
-        def validate_int_ge_1(number, name):
+        def validate_int_ge(number, name, threshold=1):
             assert isinstance(
                 number, int
             ), f"{name} must be integer but you give {type(number)}"
-            assert number >= 1, f"{name} must be no less than 1"
+            assert number >= threshold, f"{name} must be no less than {threshold}"
 
         def validate_env():
             if env is None:
@@ -138,8 +150,9 @@ class Server:
 
         validate_env()
         assert issubclass(worker, Worker), "worker must be inherited from mosec.Worker"
-        validate_int_ge_1(num, "worker number")
-        validate_int_ge_1(max_batch_size, "maximum batch size")
+        validate_int_ge(num, "worker number")
+        validate_int_ge(max_batch_size, "maximum batch size")
+        validate_int_ge(max_wait_time, "maximum wait time", 0)
         assert (
             start_method in NEW_PROCESS_METHOD
         ), f"start method must be one of {NEW_PROCESS_METHOD}"
@@ -162,16 +175,20 @@ class Server:
 
     def _controller_args(self):
         args = []
+        self._configs.pop("dry_run")
         for key, value in self._configs.items():
-            args.extend([f"--{key}", str(value)])
+            args.extend([f"--{key}", str(value).lower()])
         for batch_size in self._worker_mbs:
             args.extend(["--batches", str(batch_size)])
-        logger.info("Mosec Server Configurations: %s", args)
+        for wait_time in self._worker_wait:
+            args.extend(
+                ["--waits", str(wait_time if wait_time else self._configs["wait"])]
+            )
+        logger.info("mosec received arguments: %s", args)
         return args
 
     def _start_controller(self):
         """Subprocess to start controller program."""
-        self._configs = vars(parse_arguments())
         if not self._server_shutdown:
             path = Path(pkg_resources.resource_filename("mosec", "bin"), "mosec")
             # pylint: disable=consider-using-with
@@ -181,7 +198,7 @@ class Server:
             self.register_daemon("controller", self._controller_process)
 
     def _terminate(self, signum, framestack):
-        logger.info("[%s] terminating server [%s] ...", signum, framestack)
+        logger.info("received signum[%s], terminating server [%s]", signum, framestack)
         self._server_shutdown = True
 
     @staticmethod
@@ -271,25 +288,23 @@ class Server:
                 ctr_exitcode = self._controller_process.poll()
                 if ctr_exitcode is not None:  # exited
                     if ctr_exitcode:  # on error
-                        logger.error(
-                            "mosec controller halted on error: %d", ctr_exitcode
-                        )
+                        logger.error("mosec service halted on error: %d", ctr_exitcode)
                     else:
-                        logger.info("mosec controller halted normally")
+                        logger.info("mosec service halted normally")
                     break
                 sleep(0.1)
 
         # shutdown coordinators
         self._coordinator_shutdown.set()
-        shutil.rmtree(self._configs["path"])
-        logger.info("mosec server exited. see you.")
+        shutil.rmtree(self._configs["path"], ignore_errors=True)
+        logger.info("mosec exited normally")
 
     def register_daemon(self, name: str, proc: subprocess.Popen):
         """Register a daemon to be monitored.
 
         Args:
-            name (str): the name of this daemon
-            proc (mp.Process): the process handle of the daemon
+            name: the name of this daemon
+            proc: the process handle of the daemon
         """
         assert isinstance(name, str), "daemon name should be a string"
         assert isinstance(
@@ -302,23 +317,31 @@ class Server:
         worker: Type[Worker],
         num: int = 1,
         max_batch_size: int = 1,
+        max_wait_time: int = 0,
         start_method: str = "spawn",
         env: Union[None, List[Dict[str, str]]] = None,
     ):
         """Sequentially appends workers to the workflow pipeline.
 
-        Arguments:
-            worker: the class you inherit from `Worker` which implements
-                the `forward` method
+        Args:
+            worker: the class you inherit from :class:`Worker<mosec.worker.Worker>`
+                which implements the :py:meth:`forward<mosec.worker.Worker.forward>`
             num: the number of processes for parallel computing (>=1)
-            max_batch_size: the maximum batch size allowed (>=1)
+            max_batch_size: the maximum batch size allowed (>=1), will enable the
+                dynamic batching if it > 1
+            max_wait_time: the maximum wait time (millisecond) for dynamic batching,
+                needs to be used with `max_batch_size` to enable the feature. If not
+                configure, will use the CLI argument `--wait` (default=10ms)
             start_method: the process starting method ("spawn" or "fork")
             env: the environment variables to set before starting the process
         """
-        self._validate_arguments(worker, num, max_batch_size, start_method, env)
+        self._validate_arguments(
+            worker, num, max_batch_size, max_wait_time, start_method, env
+        )
         self._worker_cls.append(worker)
         self._worker_num.append(num)
         self._worker_mbs.append(max_batch_size)
+        self._worker_wait.append(max_wait_time)
         self._coordinator_env.append(env)
         self._coordinator_ctx.append(start_method)
         self._coordinator_pools.append([None] * num)
@@ -326,6 +349,15 @@ class Server:
     def run(self):
         """Start the mosec model server."""
         self._validate_server()
+        if self._configs["dry_run"]:
+            DryRunner(
+                self._worker_cls,
+                self._worker_mbs,
+                self._coordinator_env,
+            ).run()
+            return
+
+        self._handle_signal()
         self._start_controller()
         try:
             self._manage_coordinators()
@@ -333,18 +365,3 @@ class Server:
         except Exception:
             logger.error(traceback.format_exc().replace("\n", " "))
         self._halt()
-
-
-@contextlib.contextmanager
-def env_var_context(env: Union[None, List[Dict[str, str]]], index: int):
-    """Manage the environment variables for a worker process."""
-    default: Dict = {}
-    try:
-        if env is not None:
-            for key, value in env[index].items():
-                default[key] = os.getenv(key, "")
-                os.environ[key] = value
-        yield None
-    finally:
-        for key, value in default.items():
-            os.environ[key] = value
